/**
 * Boilerplate: $(std_boilerplate.html)
 */
module std.error;

private
{
    import std.c.stdio;
    import std.c.stdlib;
}

private
{
    alias void function( char[] file, uint line ) assertHandlerType;
    alias void function() terminateHandlerType;

    static assertHandlerType assertHandler = null;
}

class RuntimeError : Exception
{
    this( char[] msg, Exception next = null )
    {
        super( msg, next );
    }

    this( char[] msg, char[] file, size_t line, Exception next = null )
    {
        super( msg, file, line, next );
    }
}

class LogicError : Exception
{
    this( char[] msg, Exception next = null )
    {
        super( msg, next );
    }

    this( char[] msg, char[] file, size_t line, Exception next = null )
    {
        super( msg, file, line, next );
    }
}

class ArrayBoundsError : LogicError
{
    this( char[] file, size_t line )
    {
        super( "array index out of bounds", file, line );
    }
}

class AssertError : LogicError
{
    this( char[] file, size_t line )
    {
        super( "assertion failure", file, line );
    }
}

class OutOfMemoryError : RuntimeError
{
    this( char[] file, size_t line )
    {
        super( "memory allocation failed", file, line );
    }
}

class SwitchError : LogicError
{
    this( char[] file, size_t line )
    {
        super( "no appropriate switch clause found", file, line );
    }
}

void setAssertHandler( assertHandlerType h )
{
    assertHandler = h;
}

extern (C) void onAssert( char[] file, uint line )
{
    if( assertHandler )
        assertHandler( file, line );
    else
        throw new AssertError( file, line );
}

// NOTE: currently, the rest of these functions are for internal use and
//       meant to eliminate object dependencies between the runtime and
//       the standard lib.

extern (C) void onOutOfMemory()
{
    throw cast(OutOfMemoryError) cast(void*) OutOfMemoryError.classinfo.init;
}

extern (C) void onArrayBoundsError( char[] file, size_t line )
{
    throw new ArrayBoundsError( file, line );
}

extern (C) void onSwitchError( char[] file, size_t line )
{
    throw new SwitchError( file, line );
}

extern (C) void onInvalidUtfError( char[] msg, size_t idx )
{
    throw new InvalidUtfError( msg, idx );
}

class InvalidUtfError : Exception
{
    size_t idx;

    this( char[] msg, size_t idx )
    {
        super( msg );
        this.idx = idx;
    }
}