# Makefile to build D runtime library ares.lib for Win32
# Designed to work with \dm\bin\make.exe
# Targets:
#	make
#		Same as make unittest
#	make ares.lib
#		Build ares.lib
#	make clean
#		Delete unneeded files created by build process
#	make unittest
#		Build ares.lib, build and run unit tests
# Notes:
#	This relies on LIB.EXE 8.00 or later, and MAKE.EXE 5.01 or later.

CP=copy /y

CFLAGS=-mn -6 -r -Igc
#CFLAGS=-g -mn -6 -r -Igc

DFLAGS=-release -O -inline
#DFLAGS=-g -release

TFLAGS=-O -inline
#TFLAGS=-g

CC=\bin\dm\bin\dmc
DC=\bin\dmd\bin\dmd

DOC=..\..\doc\ares

.c.obj:
	$(CC) -c $(CFLAGS) $*

.cpp.obj:
	$(CC) -c $(CFLAGS) $*

.d.obj:
	$(DC) -c $(DFLAGS) $*

.asm.obj:
	$(CC) -c $*

targets : ares

ares : ares.lib

######################################################

OBJ_STD= atomic.obj error.obj intrinsic.obj thread.obj

OBJ_STD_C= complex.obj ctype.obj errno.obj fenv.obj inttypes.obj \
    math.obj signal.obj stdarg.obj stdbool.obj stddef.obj stdint.obj  \
    stdlib.obj string.obj time.obj wctype.obj
### linking in stdio breaks i/o
# stdio.obj

OBJ_STD_C_POSIX= inttypes.obj unistd.obj

OBJ_STD_C_POSIX_SYS= types.obj

OBJ_SYS_WIN_C= com.obj process.obj windows.obj

OBJ_SYS_LINUX_C=linux.obj linuxextern.obj

######################################################

DOC_STD= $(DOC)\std\atomic.html $(DOC)\std\error.html $(DOC)\std\intrinsic.html \
         $(DOC)\std\thread.html

######################################################

ares.lib : $(OBJ_STD) $(OBJ_STD_C) $(OBJ_SYS_WIN_C)
    del ares.lib
	lib -c -n ares.lib $(OBJ_STD) $(OBJ_STD_C) $(OBJ_SYS_WIN_C)

html : $(DOCS)

######################################################

### std

atomic.obj : std\atomic.d
    $(DC) -c $(TFLAGS) -Df$(DOC)\std\atomic.html std\atomic.d

error.obj : std\error.d
    $(DC) -c $(DFLAGS) -Df$(DOC)\std\error.html std\error.d

intrinsic.obj : std\intrinsic.d
    $(DC) -c $(DFLAGS) -Df$(DOC)\std\intrinsic.html std\intrinsic.d

thread.obj : std\thread.d
	$(DC) -c $(DFLAGS) -Df$(DOC)\std\thread.html std\thread.d

### std\c

complex.obj : std\c\complex.d
	$(DC) -c $(DFLAGS) std\c\complex.d

ctype.obj : std\c\ctype.d
	$(DC) -c $(DFLAGS) std\c\ctype.d

errno.obj : std\c\errno.d
	$(DC) -c $(DFLAGS) std\c\errno.d

fenv.obj : std\c\fenv.d
	$(DC) -c $(DFLAGS) std\c\fenv.d

inttypes.obj : std\c\inttypes.d
	$(DC) -c $(DFLAGS) std\c\inttypes.d

math.obj : std\c\math.d
	$(DC) -c $(DFLAGS) std\c\math.d

signal.obj : std\c\signal.d
	$(DC) -c $(DFLAGS) std\c\signal.d

stdarg.obj : std\c\stdarg.d
	$(DC) -c $(DFLAGS) std\c\stdarg.d

stdbool.obj : std\c\stdbool.d
	$(DC) -c $(DFLAGS) std\c\stdbool.d

stddef.obj : std\c\stddef.d
	$(DC) -c $(DFLAGS) std\c\stddef.d

stdint.obj : std\c\stdint.d
	$(DC) -c $(DFLAGS) std\c\stdint.d

stdio.obj : std\c\stdio.d
	$(DC) -c $(DFLAGS) std\c\stdio.d

stdlib.obj : std\c\stdlib.d
	$(DC) -c $(DFLAGS) std\c\stdlib.d

string.obj : std\c\string.d
	$(DC) -c $(DFLAGS) std\c\string.d

time.obj : std\c\time.d
	$(DC) -c $(DFLAGS) std\c\time.d

wctype.obj : std\c\wctype.d
	$(DC) -c $(DFLAGS) std\c\wctype.d

### sys\windows\c

com.obj : sys\windows\c\com.d
	$(DC) -c $(DFLAGS) sys\windows\c\com.d

process.obj : sys\windows\c\process.d
	$(DC) -c $(DFLAGS) sys\windows\c\process.d

windows.obj : sys\windows\c\windows.d
	$(DC) -c $(DFLAGS) sys\windows\c\windows.d

######################################################

$(DOC)\std\atomic.html : std.ddoc std\atomic.d
    $(DC) -c -o- $(TFLAGS) -Df$(DOC)\std\atomic.html std.ddoc std\atomic.d

$(DOC)\std\error.html : std.ddoc std\error.d
    $(DC) -c -o- $(DFLAGS) -Df$(DOC)\std\error.html std.ddoc std\error.d

$(DOC)\std\intrinsic.html : std.ddoc std\intrinsic.d
    $(DC) -c -o- $(DFLAGS) -Df$(DOC)\std\intrinsic.html std.ddoc std\intrinsic.d

$(DOC)\std\thread.html : std.ddoc std\thread.d
	$(DC) -c -o- $(DFLAGS) -Df$(DOC)\std\thread.html std.ddoc std\thread.d

######################################################

clean :
	del $(OBJ_STD) $(OBJ_STD_C) $(OBJ_SYS_WIN_C)
	del ares.lib
	del $(DOCS)

install :
	$(CP) ares.lib ..\..\lib\.
